# Generated by Django 2.0 on 2019-01-16 15:18

import django.contrib.postgres.fields
import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.db.models.deletion
import django_orion_model.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ContextBroker',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50, unique=True)),
                ('url', models.URLField()),
                ('oauth_enable', models.BooleanField(default=False)),
                ('oauth_url', models.URLField(blank=True)),
                ('oauth_user', models.CharField(blank=True, max_length=50)),
                ('oauth_password', models.CharField(blank=True, max_length=50)),
                ('oauth_client_id', models.CharField(blank=True, max_length=50)),
                ('oauth_client_secret', models.CharField(blank=True, max_length=50)),
                ('oauth_access_token', models.CharField(blank=True, max_length=50)),
                ('oauth_refresh_token', models.CharField(blank=True, max_length=50)),
            ],
        ),
        migrations.CreateModel(
            name='OrionEntity',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('orion_id', models.CharField(max_length=1011, unique=True)),
                ('orion_type', models.CharField(max_length=1011)),
                ('orion_data', django.contrib.postgres.fields.jsonb.JSONField(blank=True, default=dict)),
                ('orion_updated', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=100), blank=True, default=list, size=None)),
                ('orion_expiration', models.DateTimeField(blank=True, default=django_orion_model.models._expire_time)),
                ('orion_status', models.CharField(blank=True, choices=[('CREATING', 'Creating'), ('CREATED', 'Created'), ('WRITING', 'Pending writing'), ('READING', 'Pending reading'), ('OK', 'Ok'), ('REFRESH', 'Await refresh'), ('REWRITING', 'Await rewrite'), ('OFFLINE', 'Offline'), ('MISSING', 'Missing')], default='created', max_length=50)),
                ('orion_error', models.TextField(blank=True)),
            ],
        ),
        migrations.CreateModel(
            name='Service',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50, unique=True)),
                ('path', models.CharField(max_length=65000)),
                ('context_broker', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_orion_model.ContextBroker')),
            ],
        ),
        migrations.CreateModel(
            name='ServicePath',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50, unique=True)),
                ('path', models.CharField(max_length=65000)),
                ('service', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_orion_model.Service')),
            ],
        ),
        migrations.AddField(
            model_name='orionentity',
            name='orion_service_path',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_orion_model.ServicePath'),
        ),
    ]



import os

from django.apps import apps as django_apps
from django.views.generic.detail import SingleObjectTemplateResponseMixin
from django.views.generic.edit import ModelFormMixin
from django.views.generic.list import MultipleObjectMixin
from django_orion_model.models import ServicePath, ContextBroker, OrionEntity
from logging import getLogger


logger = getLogger(__name__)


def select_subclass(base_model, orion_type, orion_sub_type):
    class_to_use = None
    subclasses = [subclass for subclass in base_model.__subclasses__()]
    while subclasses:
        cls = subclasses.pop()
        if cls.orion_type == orion_type:
            if (orion_sub_type is None) or orion_sub_type == cls.ORION_SUB_TYPE:
                class_to_use = cls
                break
            elif cls.ORION_SUB_TYPE is None:
                class_to_use = cls
        subclasses.extend(subclass for subclass in cls.__subclasses__())
    class_to_use = class_to_use or base_model
    return class_to_use


def get_context_broker_and_service_path(context_broker, service_path):
    service_path = ServicePath.objects.get(name=service_path)
    context_broker = ContextBroker.objects.get(name=context_broker)
    return context_broker, service_path


def get_types(base_model, root=False):
    if root:
        subclasses = [base_model]
    else:
        try:
            subclasses = base_model.__subclasses__()
        except AttributeError as ex:
            logger.warning("While unraveling base class: %s", ex.message)
            return []
    types = []
    while subclasses:
        try:
            subclass = subclasses.pop()
            subclasses.extend(subclass.__subclasses__())
            types.append({
                "app": subclass._meta.app_label,
                "model": subclass.__name__,
                "orion_type": subclass.ORION_TYPE,
                "caption": subclass._meta.verbose_name
            })
        except AttributeError as ex:
            logger.warning("While unraveling subclasses: %s", ex.message)

    return types


class DynamicModelView(ModelFormMixin):

    app_label = None
    model_name = None

    model = None
    base_model = None

    def get_form_class(self):
        if self.model is None:
            self.app_label = self.kwargs.get("app",  self.request.GET.get("app", self.base_model._meta.app_label))
            self.model_name = self.kwargs.get("model", self.request.GET.get("model", self.base_model._meta.model_name))
            self.model = django_apps.get_model(self.app_label, self.model_name)
        return super().get_form_class()


class DynamicTemplate(SingleObjectTemplateResponseMixin):
    template_path = ""
    model = None
    base_model = None
    name_template = "{}{}/{}.html"

    def get_template_names(self):
        names = []

        file_name = self.name_template.format(self.template_path, self.model._meta.app_label, self.model._meta.model_name)
        # orion_manager/forms/waste_collection/waste.html
        names.append(file_name)
        # orion_manager/forms/waste.html
        names.append(os.path.join(self.template_path, file_name))
        backup_models = [p for p in self.model._meta.parents if issubclass(p, self.base_model)]
        while backup_models:
            next_model = backup_models.pop()

            # orion_manager/forms/waste_collection/waste.html
            file_name = self.name_template.format(
                self.template_path, next_model._meta.app_label, next_model._meta.model_name)
            names.append(file_name)
            # Add parents while are subclass of parent node and not yet included
            backup_models.extend(
                p for p in next_model._meta.parents if issubclass(p, self.base_model) if p not in backup_models)
        return names


class DynamicQuerySet(MultipleObjectMixin):

    def get_queryset(self):
        context_broker, service_path = get_context_broker_and_service_path(
            context_broker=self.request.GET.get("context_broker", "default"),
            service_path=self.request.GET.get("service_path", "real"))

        orion_type = self.request.GET.get("ORION_TYPE", OrionEntity.orion_type)
        orion_sub_type = self.request.GET.get("ORION_SUB_TYPE", OrionEntity.ORION_SUB_TYPE)
        class_to_use = select_subclass(self.model, orion_type, orion_sub_type)
        self.queryset = class_to_use.objects.filter(
            orion_context_broker=context_broker, orion_service_path=service_path)

        return super().get_queryset()


from django.test import TestCase
from unittest.mock import Mock, patch


from waste_collection.models import KPI
from orion_manager.models import OrionEntity


class TestOrionEntity(TestCase):

    @patch.object(OrionEntity._orion, "get", Mock())
    def test_from_db(self):
        OrionEntity.objects.get(pk="")
        OrionEntity.from_db(None, ["id"], ["WE1212"])
        OrionEntity._orion.get.assert_called_with(element_id="WE1212")

    @patch.object(OrionEntity._orion, "patch", Mock())
    def test_save(self):
        orion_entity = OrionEntity()
        orion_entity.orion_id = "WE1212"
        orion_entity.type = "sensor"
        orion_entity.save()
        orion_entity._orion.patch.assert_not_called()
        orion_entity.data = {
            "id": "WE1212",
            "type": "Fake",
            "temperature": {
                "value": 23,
                "type": "Float"
            },
            "pressure": {
                "value": 720,
                "type": "Integer"
            }}
        orion_entity.updated = "temperature, pressure"

        orion_entity.save()
        orion_entity._orion.patch.assert_called_with(element_id="WE1212", **{
            "temperature": {
                "value": 23,
                "type": "Float"
            },
            "pressure": {
                "value": 720,
                "type": "Integer"
            }}
            )

    def test_global_property(self):

        orion_entity2 = KPI()
        orion_entity3 = KPI()
        orion_entity2.name = "Babar"
        self.assertNotEqual(orion_entity2.name, orion_entity3.name, "Global property error")

    def test_save_to_orion(self):
        self.fail()

    def test_update_from_orion(self):
        self.fail()

    def test_Now(self):
        self.fail()



from django.views.generic.detail import SingleObjectMixin
from django.core.exceptions import ImproperlyConfigured
from logging import getLogger


logger = getLogger(__name__)


class DynamicSingleObjectMixin(SingleObjectMixin):
    """Provides de capability to load a object  by a specific subclass stored in a """
    model = None
    base_model = None
    subclass_field = "subclass"

    def get_queryset(self):
        if self.base_model:
            return self.base_model._default_manager.all()
        else:
            raise ImproperlyConfigured(
                "%(cls)s is missing a QuerySet. Define "
                "%(cls)s.model, %(cls)s.queryset, or override "
                "%(cls)s.get_queryset()." % {
                    'cls': self.__class__.__name__
                }
            )

    def get_object(self, queryset=None):
        obj = super().get_object(queryset)
        try:
            return obj.subclassed()
        except AttributeError:
            logger.debug("%s is not subclassed", obj.__class__.__name__)
            return obj
